==> java-squid-2.7-sources.jar: org/sonar/java/bytecode/VirtualMethodsLinker.java <==
/*
 * SonarQube Java
 * Copyright (C) 2012 SonarSource
 * dev@sonar.codehaus.org
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.java.bytecode;

import org.sonar.java.bytecode.asm.AsmClass;
import org.sonar.java.bytecode.asm.AsmMethod;

public class VirtualMethodsLinker {

  public void process(AsmMethod method) {
    if (!method.isBodyLoaded() && method.isUsed()) {
      AsmMethod implementation = findImplementation(method.getParent(), method.getKey());
      if (implementation != null) {
        implementation.setUsed(true);
        method.linkTo(implementation);
      }
    }
  }

  private AsmMethod findImplementation(AsmClass asmClass, String methodKey) {
    AsmMethod method = asmClass.getMethod(methodKey);
    if (method != null && method.isBodyLoaded()) {
      return method;
    }
    if (asmClass.getSuperClass() != null) {
      return findImplementation(asmClass.getSuperClass(), methodKey);
    }
    return null;
  }

}

==> java-squid-3.6-sources.jar: org/sonar/java/bytecode/VirtualMethodsLinker.java <==
/*
 * SonarQube Java
 * Copyright (C) 2012 SonarSource
 * sonarqube@googlegroups.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.java.bytecode;

import org.sonar.java.bytecode.asm.AsmClass;
import org.sonar.java.bytecode.asm.AsmMethod;

public class VirtualMethodsLinker {

  public void process(AsmMethod method) {
    if (!method.isBodyLoaded() && method.isUsed()) {
      AsmMethod implementation = findImplementation(method.getParent(), method.getKey());
      if (implementation != null) {
        implementation.setUsed(true);
        method.linkTo(implementation);
      }
    }
  }

  private static AsmMethod findImplementation(AsmClass asmClass, String methodKey) {
    AsmMethod method = asmClass.getMethod(methodKey);
    if (method != null && method.isBodyLoaded()) {
      return method;
    }
    if (asmClass.getSuperClass() != null) {
      return findImplementation(asmClass.getSuperClass(), methodKey);
    }
    return null;
  }

}
